<?php

define('RBS_PROD_URL' , 'https://pay.alfabank.ru/payment/rest/');
define('RBS_TEST_URL' , 'https://alfa.rbsuat.com/payment/rest/');
define('RBS_PROD_URL_ALT' , 'https://payment.alfabank.ru/payment/rest/');
define('RBS_PROD_URL_ALT_PREFIX' , 'r-');
?><?php
class nc_payment_system_rbspayment extends nc_payment_system
{

    static public $rbs_vats = array(
        'none' => 0,
        '0' => 1,
        '10' => 2,
        '18' => 3,
        '20' => 6
    );
    public $version = '2.6.3';
    protected $automatic = TRUE;
    protected $accepted_currencies = array('RUB', 'RUR','BYN', 'EUR', 'USD');
    protected $currency_map = array('RUR' => 'RUB');
    protected $settings = array(
        'userName' => null,
        'password' => null,
        'test_mode' => 'yes',
        'two-stage' => 'no',
        'successUrl' => null,
        'failUrl' => null,

        'send_order' => 'no',
        'taxSystem' => null,
        'FFDVersion' => 'v1_05',
        'paymentMethod' => 1,
        'paymentMethod_delivery' => 1,
        'paymentObject' => 4,
        'write_log' => 'no',
    );

    protected $request_parameters = array();
    protected $callback_response = array(
        'formUrl' => null,
        'orderNumber' => null
    );

    public function execute_payment_request(nc_payment_invoice $invoice)
    {
        $return_url = nc_get_scheme() . '://' . $_SERVER['HTTP_HOST'] . nc_module_path('payment') .
            'callback.php?paySystem=nc_payment_system_rbspayment&invoice_id=' . $invoice->get_id();

        $amount = $invoice->get_amount("%0.2F");
        $order_id = $invoice->get_id();
        $invoice_id = $invoice->get_id();

        $data = array(
            'userName' => $this->settings['userName'],
            'password' => $this->settings['password'],
            'orderNumber' => $invoice_id . "_" . time(),
            'amount' => $amount * 100,
            'returnUrl' => $return_url,
            'jsonParams' => json_encode(
                array(
                    'CMS:' => 'NetCat 5.x',
                    'Module-Version: ' => $this->version
                )
            ),
        );

        //collect order data
        if ((defined('RBS_ENABLE_FISCALE_OPTIONS') && RBS_ENABLE_FISCALE_OPTIONS === true) && $this->settings['send_order'] == 'yes') {

            $data['taxSystem'] = empty($this->settings['taxSystem']) ? 0 : $this->settings['taxSystem'];

            $items = array();
            $itemsCnt = 1;

            $netshop = nc_netshop::get_instance();
            $order = $netshop->load_order($order_id);
            $collection = $order->get_items();

            $sellReceipt = $invoice->get_sell_receipt();
            foreach ($sellReceipt->get_items() as $i) {
                if ($i['source_item_id']) {

                    $item = array();
                    $product_price = round((float)$i->get('item_price') * 100);

                    $item['positionId'] = $itemsCnt++;
                    $item['name'] = mb_substr($i['name'], 0, 64);
                    $item['quantity'] = array(
                        'value' => $i->get('qty'),
                        'measure' => $this->settings['FFDVersion'] == "v1_05" ? RBS_MEASUREMENT_NAME : RBS_MEASUREMENT_CODE
                    );
                    $item['itemAmount'] = round($i->get('qty') * (float)$i->get('item_price') * 100);
                    $item['itemCode'] = $i->get('id');
                    $item['tax'] = array(
                        'taxType' => nc_array_value(self::$rbs_vats, $i->get('vat_rate'), self::$rbs_vats['none']),
                    );
                    $item['itemPrice'] = $product_price;

                    // FFD 1.05 added
//                    if ($this->settings['ffdVersion'] == "v105") {
                    $attributes = array();
                    $attributes[] = array(
                        "name" => "paymentMethod",
                        "value" => $this->settings['paymentMethod']
                    );
                    $attributes[] = array(
                        "name" => "paymentObject",
                        "value" => $this->settings['paymentObject']
                    );

                    $item['itemAttributes']['attributes'] = $attributes;
//                    }
                    $items[] = $item;
                }
            }


            // DELIVERY POSITION
            if (isset($order['DeliveryCost']) && $order['DeliveryCost']) {
                $price = round((float)$order['DeliveryCost'] * 100);
                $itemShipment['positionId'] = $itemsCnt;
                $itemShipment['name'] = mb_substr($order->get_delivery_method()['name'], 0, 64);
                $itemShipment['quantity'] = array(
                    'value' => 1,
                    'measure' => $this->settings['FFDVersion'] == "v1_05" ? RBS_MEASUREMENT_NAME : RBS_MEASUREMENT_CODE
                );
                $itemShipment['itemAmount'] = $price;
                $itemShipment['itemCode'] = 'Delivery';
                $itemShipment['tax'] = array(
                    'taxType' => 0 //todo
                );

                // FFD 1.05 added
//                if ($this->settings['ffdVersion'] == "v105") {
                $attributes = array();
                $attributes[] = array(
                    "name" => "paymentMethod",
                    "value" => $this->settings['paymentMethod']
                );
                $attributes[] = array(
                    "name" => "paymentObject",
                    "value" => 4
                );
                $itemShipment['itemAttributes']['attributes'] = $attributes;
//                }
                $items[] = $itemShipment;
            }

            $invoice_data = $invoice->to_array();

            $order_bundle = array(
                'orderCreationDate' => date("c"),
                'customerDetails' => array(
                    'email' => $invoice_data['customer_email'],
                    'phone' => preg_replace("/[^0-9]/", "", nc_normalize_phone_number($invoice_data['customer_phone'])),
                ),
                'cartItems' => array('items' => $items)
            );

            $data['orderBundle'] = json_encode($order_bundle);

        }

        $action_adr = RBS_TEST_URL;
        if ($this->settings['test_mode'] != 'yes') {
            $action_adr = RBS_PROD_URL;
            // for migrate to new payment gateway
            if (defined('RBS_PROD_URL_ALT') && defined('RBS_PROD_URL_ALT_PREFIX')) {
                if (substr($this->merchant, 0, strlen(RBS_PROD_URL_ALT_PREFIX)) == RBS_PROD_URL_ALT_PREFIX) {
                    $action_adr = RBS_PROD_URL_ALT;
                } else {
                    //disable callbacks for old gateway version
                }
            }
        }

        if ($this->settings['test_mode'] != 'yes') {
            $action_adr = RBS_TEST_URL;
            $gate_url = str_replace("payment/rest", "mportal-uat/mvc/public/merchant/update", $action_adr);
        } else {
            $action_adr = RBS_PROD_URL;
            $gate_url = str_replace("payment/rest", "mportal/mvc/public/merchant/update", $action_adr);
        }

        $callback_addresses_string = $return_url;
        if ($this->allowCallbacks !== false) {
            $response = $this->_updateGatewayCallback($this->merchant, $this->password, $gate_url, $callback_addresses_string);

            if (RBS_ENABLE_LOGGING === true) {
                $this->_writeLog("[callback_addresses_string]: " . $callback_addresses_string . "\n[RESPONSE]: " . $response);
            }
        }

        if ($this->settings['two-stage'] == 'yes') {
            $action_adr .= 'registerPreAuth.do';
        } else {
            $action_adr .= 'register.do';
        }


        $response = $this->execute_payment_session($data, $action_adr);

        if ($response['errorCode'] == 0) {
            header("Location: " . $response['formUrl']);
        } else {
            die("RBSPayment[ERROR]: " . $response['errorMessage']);
        }

        exit;
    }

    public function execute_payment_session($data, $action_adr)
    {
        $headers = array(
            'CMS: NetCat 6.x',
            'Module-Version: ' . $this->version,
        );
        $response = $this->_sendGatewayData($data, $action_adr, $headers);
        $this->writeLog("[REQUEST RU]: " . $action_adr . "\n[RESPONSE]: " . print_r($response, true));
        return json_decode($response, TRUE);
    }

//    function rbs_logger($data)
//    {
//        if ($data) {
//
//            $result = $data;
//            if (is_array($data) || is_object($data)) {
//                $result = "\n" . print_r($data, true);
//            }
//            $result .= "\n\n";
//            $path = dirname(__FILE__) . '/rbspayment.log';
//            error_log($result, 3, $path);
//            return true;
//        }
//        return false;
//    }

    function _writeLog($var, $info = true)
    {
        $information = "";
        if ($var) {
            if ($info) {
                $information = "\n\n";
                $information .= str_repeat("-=", 64);
                $information .= "\nDate: " . date('Y-m-d H:i:s');
            }

            $result = $var;
            if (is_array($var) || is_object($var)) {
                $result = "\n" . print_r($var, true);
            }
            $result .= "\n\n";
            $path = dirname(__FILE__) . '/rbspayment_' . date('Y-m') . '.log';
            error_log($information . $result, 3, $path);
            return true;
        }
        return false;
    }

    /**
     * @param nc_payment_invoice $invoice
     */
    public function on_response(nc_payment_invoice $invoice = null)
    {
        $invoice_status = $invoice->get('status');
        $success_url = $this->get_setting('successUrl') ?: '/';
        $fail_url = $this->get_setting('failUrl') ?: '/';

        if ($invoice_status === nc_payment_invoice::STATUS_CALLBACK_ERROR) {
            $this->on_payment_failure($invoice);
            header('Location: ' . $fail_url);
        } else {
            $this->on_payment_success($invoice);
            header('Location: ' . $success_url);
        }
    }

    /**
     *
     */
    public function validate_payment_request_parameters()
    {

    }

    /**
     * @param nc_payment_invoice $invoice
     */
    public function validate_payment_callback_response(nc_payment_invoice $invoice = null)
    {

        if (!$invoice) {
            $response_description = 'Invoice ' . ($this->get_response_value('invoice_id')) . ' does not exist';
            $this->respond_with_xml('error', $response_description);
        }

        $data = array(
            'userName' => $this->settings['userName'],
            'password' => $this->settings['password'],
            'orderId' => $this->get_response_value('orderId'),
        );

//        $action_adr = ($this->settings['test_mode'] == 'yes') ? self::RBS_TEST_URL : self::RBS_PROD_URL;

        $action_adr = RBS_TEST_URL;
        if ($this->settings['test_mode'] != 'yes') {
            $action_adr = RBS_PROD_URL;
            // for migrate to new payment gateway
            if (defined('RBS_PROD_URL_ALT') && defined('RBS_PROD_URL_ALT_PREFIX')) {
                if (substr($this->merchant, 0, strlen(RBS_PROD_URL_ALT_PREFIX)) == RBS_PROD_URL_ALT_PREFIX) {
                    $action_adr = RBS_PROD_URL_ALT;
                } else {
                    //not used this $this->allowCallbacks = false;
                }
            }
        }

        $result = $this->_sendGatewayData($data, $action_adr . "getOrderStatusExtended.do");
        $response = json_decode($result, TRUE);


        $success_url = $this->get_setting('successUrl') ?: '/';
        $fail_url = $this->get_setting('failUrl') ?: '/';

        $orderStatus = $response['orderStatus'];

        if ($orderStatus != '1' && $orderStatus != '2') {
            $this->add_error($response['errorMessage']);
            $this->on_payment_rejected($invoice);

            header("Location: " . $fail_url);

        } else {
            $nc_order_number = explode("_", $response['orderNumber']);
            $invoice_id = intval($nc_order_number[0]);

            $invoice = $this->load_invoice($invoice_id);
            $this->on_payment_success($invoice);

            header("Location: " . $success_url);
        }
    }


    public function load_invoice_on_callback()
    {
        return $this->load_invoice($this->get_response_value('invoice_id'));
    }

    public function _updateGatewayCallback($login, $password, $action_address, $callback_addresses_string)
    {
        $headers = array(
            'Content-Type:application/json',
            'Authorization: Basic ' . base64_encode($login . ":" . $password)
        );
        $data['callbacks_enabled'] = true;
        $data['callback_addresses'] = $callback_addresses_string;
        $data['callback_operations'] = "deposited,approved,declinedByTimeout";

        $response = $this->_sendGatewayData(json_encode($data), $action_address, $headers);
        return $response;
    }

    public function _sendGatewayData($data, $action_address, $headers = array())
    {

        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_HTTPHEADER => $headers,
            CURLOPT_VERBOSE => true,
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_URL => $action_address,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => $data,
            //                CURLOPT_ENCODING, "gzip",
        ));
        $response = curl_exec($ch);
        curl_close($ch);

        return $response;
    }

}